@@ -6,10 +6,62 @@ https://github.com/agoragames/kairos/blob/master/LICENSE.txt
 from .exceptions import *
 from .timeseries import *
 
-from sqlalchemy import Table, Column, Integer, String, Float, MetaData, UniqueConstraint
+from sqlalchemy.types import TypeEngine
+from sqlalchemy import Table, Column, BigInteger, Integer, String, Unicode, Text, LargeBinary, Float, Boolean, Time, Date, DateTime, Numeric, MetaData, UniqueConstraint
 from sqlalchemy.sql import select, update, insert, distinct, asc, desc, and_, or_, not_
 
 import time
+from datetime import date, datetime
+from datetime import time as time_type
+from decimal import Decimal
+
+# Test python3 compatibility
+try:
+  x = long(1)
+except NameError:
+  long = int
+try:
+  x = unicode('foo')
+except NameError:
+  unicode = str
+
+TYPE_MAP = {
+  str         : String,
+  'str'       : String,
+  'string'    : String,
+
+  unicode     : Unicode,  # works for py3 too
+  'unicode'   : Unicode,
+
+  float       : Float,
+  'float'     : Float,
+
+  int         : Integer,
+  'int'       : Integer,
+  'integer'   : Integer,
+
+  long        : BigInteger, # works for py3 too
+  'long'      : BigInteger,
+  'int64'     : BigInteger,
+
+  bool        : Boolean,
+  'bool'      : Boolean,
+  'boolean'   : Boolean,
+
+  date        : Date,
+  'date'      : Date,
+  datetime    : DateTime,
+  'datetime'  : DateTime,
+  time_type   : Time,
+  'time'      : Time,
+
+  Decimal     : Numeric,
+  'decimal'   : Numeric,
+
+  'text'      : Text,
+  'clob'      : Text,
+  'blob'      : LargeBinary,
+}
 
 class SqlBackend(Timeseries):
   
@@ -31,6 +83,35 @@ class SqlBackend(Timeseries):
     Initialize the sql backend after timeseries has processed the configuration.
     '''
     self._metadata = MetaData()
+    self._str_length = kwargs.get('string_length',255)
+    self._txt_length = kwargs.get('text_length', 32*1024)
+    
+    vtype = kwargs.get('value_type', float)
+    if vtype in TYPE_MAP:
+      self._value_type = TYPE_MAP[vtype]
+      if self._value_type == String:
+        self._value_type = String(self._str_length)
+      elif self._value_type == Text:
+        self._value_type = Text(self._txt_length)
+      elif self._value_type == LargeBinary:
+        self._value_type = LargeBinary(self._txt_length)
+
+    elif issubclass(vtype, TypeEngine):
+      if vtype == String:
+        self._value_type = String(self._str_length)
+      elif vtype == Text:
+        self._value_type = Text(self._txt_length)
+      elif vtype == LargeBinary:
+        self._value_type = LargeBinary(self._txt_length)
+
+    elif isinstance(vtype, TypeEngine):
+      self._value_type = vtype
+
+    else:
+      raise ValueError("Unsupported type '%s'"%(vtype))
+
+    self._table_name = kwargs.get('table_name', self._table_name)
+
     super(SqlBackend,self).__init__(client, **kwargs)
 
   def list(self):
@@ -54,7 +135,7 @@ class SqlBackend(Timeseries):
           self._table.c.name==name,
           self._table.c.interval==interval
         )
-      ).order_by( asc(self._table.c.i_time) )
+      ).order_by( asc(self._table.c.i_time) ).limit(1)
       rval[interval]['first'] = config['i_calc'].from_bucket(
         connection.execute(stmt).first()['i_time'] )
       
@@ -63,7 +144,7 @@ class SqlBackend(Timeseries):
           self._table.c.name==name,
           self._table.c.interval==interval
         )
-      ).order_by( desc(self._table.c.i_time) )
+      ).order_by( desc(self._table.c.i_time) ).limit(1)
       rval[interval]['last'] = config['i_calc'].from_bucket(
         connection.execute(stmt).first()['i_time'] )
 
@@ -178,17 +259,17 @@ class SqlBackend(Timeseries):
 class SqlSeries(SqlBackend, Series):
   
   def __init__(self, *a, **kwargs):
-    # TODO: Support kwargs so that we support more than just floats
     # TODO: define indices
     # TODO: optionally create separate tables for each interval, like mongo?
+    self._table_name = 'series'
     super(SqlSeries,self).__init__(*a, **kwargs)
-    self._table = Table('series', self._metadata,
-      Column('name', String(255), nullable=False),      # stat name
-      Column('interval', String(255), nullable=False),  # interval name
+    self._table = Table(self._table_name, self._metadata,
+      Column('name', String(self._str_length), nullable=False),      # stat name
+      Column('interval', String(self._str_length), nullable=False),  # interval name
       Column('insert_time', Float, nullable=False),     # to preserve order
       Column('i_time', Integer, nullable=False),        # interval timestamp
       Column('r_time', Integer, nullable=True),         # resolution timestamp
-      Column('value', Float, nullable=False)            # datas
+      Column('value', self._value_type, nullable=False)            # datas
     )
     self._metadata.create_all(self._client)
   
@@ -238,16 +319,16 @@ class SqlSeries(SqlBackend, Series):
 class SqlHistogram(SqlBackend, Histogram):
   
   def __init__(self, *a, **kwargs):
-    # TODO: Support kwargs so that we support more than just floats
     # TODO: define indices
     # TODO: optionally create separate tables for each interval, like mongo?
+    self._table_name = 'histogram'
     super(SqlHistogram,self).__init__(*a, **kwargs)
-    self._table = Table('histogram', self._metadata,
-      Column('name', String(255), nullable=False),      # stat name
-      Column('interval', String(255), nullable=False),  # interval name
+    self._table = Table(self._table_name, self._metadata,
+      Column('name', String(self._str_length), nullable=False),      # stat name
+      Column('interval', String(self._str_length), nullable=False),  # interval name
       Column('i_time', Integer, nullable=False),        # interval timestamp
       Column('r_time', Integer, nullable=True),         # resolution timestamp
-      Column('value', Float, nullable=False),           # histogram keys
+      Column('value', self._value_type, nullable=False),           # histogram keys
       Column('count', Integer, nullable=False),         # key counts
 
       # Use a constraint for transaction-less insert vs update
@@ -325,13 +406,13 @@ class SqlHistogram(SqlBackend, Histogram):
 class SqlCount(SqlBackend, Count):
   
   def __init__(self, *a, **kwargs):
-    # TODO: Support kwargs so that we support more than just floats
     # TODO: define indices
     # TODO: optionally create separate tables for each interval, like mongo?
+    self._table_name = 'count'
     super(SqlCount,self).__init__(*a, **kwargs)
-    self._table = Table('count', self._metadata,
-      Column('name', String(255), nullable=False),      # stat name
-      Column('interval', String(255), nullable=False),  # interval name
+    self._table = Table(self._table_name, self._metadata,
+      Column('name', String(self._str_length), nullable=False),      # stat name
+      Column('interval', String(self._str_length), nullable=False),  # interval name
       Column('i_time', Integer, nullable=False),        # interval timestamp
       Column('r_time', Integer, nullable=True),         # resolution timestamp
       Column('count', Integer, nullable=False),         # key counts
@@ -409,16 +490,16 @@ class SqlCount(SqlBackend, Count):
 class SqlGauge(SqlBackend, Gauge):
   
   def __init__(self, *a, **kwargs):
-    # TODO: Support kwargs so that we support more than just floats
     # TODO: define indices
     # TODO: optionally create separate tables for each interval, like mongo?
+    self._table_name = 'gauge'
     super(SqlGauge,self).__init__(*a, **kwargs)
-    self._table = Table('gauge', self._metadata,
-      Column('name', String(255), nullable=False),      # stat name
-      Column('interval', String(255), nullable=False),  # interval name
+    self._table = Table(self._table_name, self._metadata,
+      Column('name', String(self._str_length), nullable=False),      # stat name
+      Column('interval', String(self._str_length), nullable=False),  # interval name
       Column('i_time', Integer, nullable=False),        # interval timestamp
       Column('r_time', Integer, nullable=True),         # resolution timestamp
-      Column('value', Float, nullable=False),           # key counts
+      Column('value', self._value_type, nullable=False),           # key counts
 
       # Use a constraint for transaction-less insert vs update
       UniqueConstraint('name', 'interval', 'i_time', 'r_time', name='unique_count')
